//*========================================================================================*
//========================================================================================
// Project: Cart Battery Monitor - Read Battery Voltages
// Author:  Jim Abraham
// Date:    1-31-16
// Version: V1.0
// Revisions: 
//========================================================================================
//*========================================================================================*
// number of analog samples to take per reading
#define NUM_SAMPLES 10

int sum = 0;                    // sum of samples taken
unsigned char sample_count = 0; // current sample number
float voltage = 0.0;            // calculated voltage
float Vout;
int A0_Channel = 0;
int bat1v, bat2v, bat3v, bat4v, bat5v, bat6v;
int RB1_RELAY=2;
int RB2_RELAY=3;
int RB3_RELAY=4;
int RB4_RELAY=5;
int RB5_RELAY=6;
int RB6_RELAY=7;
int RRT_val;

//================================================================================
// SET-UP
//================================================================================
void setup() 
{
// setup analog inputs
  pinMode(A0, INPUT);
  
// Set all unused analog inputs to 
  pinMode(A1, OUTPUT);
  digitalWrite(A1, LOW);
  pinMode(A2, OUTPUT);
  digitalWrite(A2, LOW);
  pinMode(A3, OUTPUT);
  digitalWrite(A3, LOW);
  pinMode(A4, OUTPUT);
  digitalWrite(A4, LOW);
  pinMode(A5, OUTPUT);
  digitalWrite(A5, LOW);

// LED
  pinMode(13, OUTPUT); 
  
// RELAYS 1-6
 pinMode(2, OUTPUT);
 pinMode(3, OUTPUT);
 pinMode(4, OUTPUT);
 pinMode(5, OUTPUT);
 pinMode(6, OUTPUT);
 pinMode(7, OUTPUT);

 // RESET TIME PB
 pinMode(9, INPUT_PULLUP); // Send R to other board to reset Run Timer.

  Serial.begin(9600);   // serial communication set-up.
  delay(2000);
}

//================================================================================
// MAIN LOOP
//================================================================================
void loop() 
{
  Read_Battery_Voltages();  // read/send battery voltages every 30 seconds.
  Read_Reset_Runtime_PB(); // is low send "R" else send "C" R=Reset, C=Continue
  Send_Bat_Voltages();
  delay(18000);
}


//================================================================================
// ROUNTINES
//================================================================================

void Read_Reset_Runtime_PB()
{
  RRT_val = digitalRead(9);   // read the input pin
}

//=======================================================================
void Read_Battery_Voltages()
//=======================================================================
{
  digitalWrite(13, HIGH);
  A0_Channel = 0;
  
//-----------------------------------------------------------------------
// Read Battery #1 Voltage
//-----------------------------------------------------------------------
  digitalWrite(RB1_RELAY, HIGH);  // D2
  delay(1000);                   
  Read_Bat_Voltages();
  bat1v = (int)voltage;
  digitalWrite(RB1_RELAY, LOW);   // Turn Off Relay
  digitalWrite(13, LOW);
  delay(1000);
  
//-----------------------------------------------------------------------
// Read Battery #2 Voltage
//-----------------------------------------------------------------------
  digitalWrite(RB2_RELAY, HIGH); 
  digitalWrite(13, HIGH);  
  delay(1000);   
  Read_Bat_Voltages();
  bat2v = (int)voltage;
  digitalWrite(RB2_RELAY, LOW);   // Turn Off Relay
  digitalWrite(13, LOW);
  delay(1000);
  
//-----------------------------------------------------------------------
// Read Battery #3 Voltage
//-----------------------------------------------------------------------
  digitalWrite(RB3_RELAY, HIGH);   
  digitalWrite(13, HIGH);
  delay(1000);    
  Read_Bat_Voltages();
  bat3v = (int)voltage;
  digitalWrite(RB3_RELAY, LOW);   // Turn Off Relay
  digitalWrite(13, LOW);
  delay(1000);
  
//-----------------------------------------------------------------------
// Read Battery #4 Voltage  
//-----------------------------------------------------------------------
  digitalWrite(RB4_RELAY, HIGH);   
  digitalWrite(13, HIGH);
  delay(1000);   
  Read_Bat_Voltages();
  bat4v = (int)voltage;
  digitalWrite(RB4_RELAY, LOW);   // Turn Off Relay
  digitalWrite(13, LOW);
  delay(1000);
  
//-----------------------------------------------------------------------
// Read Battery #5 Voltage
//-----------------------------------------------------------------------
  digitalWrite(RB5_RELAY, HIGH);  
  digitalWrite(13, HIGH); 
  delay(1000);  
  Read_Bat_Voltages();
  bat5v = (int)voltage;
  digitalWrite(RB5_RELAY, LOW);   // Turn Off Relay
  digitalWrite(13, LOW);
  delay(1000);
  
//-----------------------------------------------------------------------
 // Read Battery #6 Voltage
//-----------------------------------------------------------------------
  digitalWrite(RB6_RELAY, HIGH);   
  digitalWrite(13, HIGH);
  delay(1000);     
  Read_Bat_Voltages();
  bat6v = (int)voltage;
  digitalWrite(RB6_RELAY, LOW);   // Turn Off Relay
  delay(1000);
}

//=======================================================================
void Read_Bat_Voltages()
//=======================================================================
{
// take a number of analog samples and add them up
  while (sample_count < NUM_SAMPLES) 
      {
        sum += analogRead(A0_Channel);
        sample_count++;
        delay(10);
      }
// 5/1023 = v/adc counts
// v = (5 * adc counts) / 1023
// v = adc counts (5.0 / 1023.0)
  voltage = (((float)sum / (float)NUM_SAMPLES) * (5.0 / 1023.0));

// 9v/5v = 1.8 max vin = 9v. so when adc =5v, change to 9v.
  float voltage_bias=1.8;  // = 9vdc max
  Vout = voltage * 100 * voltage_bias; // x100 for int xxx
  voltage = Vout;
  //Vout=Vin*(R2/(R1+R2))
  //  Vout = voltage * (10000/(10000 + 15000));
  
  sample_count = 0;
  sum = 0;
}

//=======================================================================
void Send_Bat_Voltages()
//=======================================================================
{
  Serial.println("B");    // BEGIN
  Serial.println(bat1v);
  Serial.println(bat2v);
  Serial.println(bat3v);
  Serial.println(bat4v);
  Serial.println(bat5v);
  Serial.println(bat6v);
  if (RRT_val==0){Serial.println("55");}  //reset
  if (RRT_val==1){Serial.println("0");}   //ok
  Serial.println("9");    // END

  digitalWrite(13, LOW);
}

