//*========================================================================================*
//========================================================================================
// Project: Cart Battery Monitor (CBM)
// Author:  Jim Abraham
// Date:    1-9-16
// Version: V1.0
// Revisions: 
//========================================================================================
//*========================================================================================*
// IMPORTANT: Adafruit_TFTLCD LIBRARY MUST BE SPECIFICALLY
// CONFIGURED FOR EITHER THE TFT SHIELD OR THE BREAKOUT BOARD.
// SEE RELEVANT COMMENTS IN Adafruit_TFTLCD.h FOR SETUP.
// Modified for SPFD5408 Library by Joao Lopes
// Version 0.9.2 - Rotation for Mega and screen initial
//========================================================================================

#include <SPFD5408_Adafruit_GFX.h>    // Core graphics library
#include <SPFD5408_Adafruit_TFTLCD.h> // Hardware-specific library
#include <SPFD5408_TouchScreen.h>
#include <EEPROM.h>
#include <SoftwareSerial.h>
#include <TimerOne.h>
// The control pins for the LCD can be assigned to any digital or
// analog pins...but we'll use the analog pins as this allows us to
// double up the pins with the touch screen (see the TFT paint example).
#define LCD_CS A3     // Chip Select goes to Analog 3
#define LCD_CD A2     // Command/Data goes to Analog 2
#define LCD_WR A1     // LCD Write goes to Analog 1
#define LCD_RD A0     // LCD Read goes to Analog 0
#define LCD_RESET A4 // Can alternately just connect to Arduino's reset pin

// Assign human-readable names to some common 16-bit color values:
#define	BLACK   0x0000
#define	BLUE    0x001F
#define	RED     0xF800
#define	GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

Adafruit_TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET);

//--------------------------------------------------------
// Variables
//--------------------------------------------------------
int BAT1V=80, BAT2V=80, BAT3V=80;
int BAT4V=80, BAT5V=80, BAT6V=80;
float BAT1Vf, BAT1Vf1, BAT2Vf,  BAT2Vf1, BAT3Vf, BAT3Vf1;
float BAT4Vf, BAT4Vf1, BAT5Vf, BAT5Vf1, BAT6Vf,  BAT6Vf1;
float TOT_BATVf,TOT_BATV_PCT, bargraph, bargraph_prev;

float b1v_prev,b2v_prev,b3v_prev,b4v_prev,b5v_prev,b6v_prev;
float t_batpct,t_batvf;
int BATVend;

int rts=0,rtm=0,rth=0;
// EEPROM
int addr, rtm_EE,rth_EE;
int rtm_EE_59_flag=0, rth_EE_99_flag=0;
int rt_reset=0;

// serial communication set-up.
int bat_voltage_ok=0;
int RRT_RC;
int intrup=0;
//========================================================================================
// Setup Rountine
//========================================================================================
void setup(void) 
{
  Serial.begin(9600);   // serial communication set-up.
  tft.reset();
  tft.begin(0x9341); // SDFP5408
  tft.setRotation(1);
  rts=0,rtm=0,rth=0;
  DISPLAY_INITIAL_Text();
  DISPLAY_Text();
  
  //GET RUN AND TOTAL TIMES FROM EEPROM
  addr=1;
  rtm_EE = EEPROM.read(addr);
//  rtm_EE=59;                           //init value
//  EEPROM.write(1, rtm_EE_value);  delay(500);//init value
  addr=5;
  rth_EE = EEPROM.read(addr);
//  rth_EE=99;                           //init value
//  EEPROM.write(5, rth_EE_value);  delay(500);//init value

  Timer1.initialize(1000000);         // initialize timer1, and set a 1 second period
  Timer1.attachInterrupt(callback);  // attaches callback() as a timer overflow interrupt

}

//========================================================================================
// Main Rountine
//========================================================================================
void loop(void) 
{
  Measure_Battery_Voltages();
  if ((bat_voltage_ok == 1) & (BATVend == 9))
    {
      if (RRT_RC==55){rt_reset=1;}
      Format_Battery_Voltages();
      Display_Battery_Voltages();
      bat_voltage_ok=0;
    }
 // else
 // {tft.setTextColor(RED); tft.setCursor(235,155);  tft.println(0.00);}
 
  if (intrup==1) 
  {
    Update_Runtime();
    intrup=0;
  }
}

//========================================================================================
// SUB-Rountines
//========================================================================================

//--------------------------------------------------------
// Update Cart Run Time on Display
//--------------------------------------------------------
void  Update_Runtime()
{
// Update Cart Run and Total Time.
 // rts = rts + 1;
  // TIME TO INCREMENT RT/TT MINUTES?
  if (rts >=60) 
  {
    rtm = rtm + 1;
    rts = 0;

    //-------------------------------------------
    // SAVE TOTAL TIME (MIN) TO EEPROM.
    //-------------------------------------------
    rtm_EE = rtm_EE + 1;
     if (rtm_EE >= 60)
      {
        rtm_EE = 0;
        rtm_EE_59_flag=1;
    //-------------------------------------------
    // SAVE TOTAL TIME (HOURS) TO EEPROM.    
    //-------------------------------------------
      rth_EE = rth_EE + 1;
      // if 99 hours clear hours digits.
      if (rth_EE > 99)
        {
          rth_EE = 0;
          rth_EE_99_flag=1;
        }
      }
      addr = 1;
      EEPROM.write(addr, rtm_EE);
      delay(100);  
      addr = 5;
      EEPROM.write(addr, rth_EE);
      delay(100);     
  }
  
//-------------------------------------------
// TIME TO INCREMENT RT HOURS?
//-------------------------------------------
  if (rtm >=60) 
  {
    rth = rth + 1;
    rtm = 0;
  }
   
//-------------------------------------------
// DISPLAY RUN TIME
//-------------------------------------------
// minutes
  if ((rts==0) && (rtm==0))
  {
    tft.setTextColor(BLACK);   tft.setCursor(250,202);  tft.println("59");
    tft.setTextColor(YELLOW);  tft.setCursor(250,202);  tft.println("0");
  }
  else
  {
    tft.setTextColor(BLACK);   tft.setCursor(250,202);  tft.println(rtm-1);
    tft.setTextColor(YELLOW);  tft.setCursor(250,202);  tft.println(rtm);
  }
  
  // hours
  tft.setTextColor(BLACK);   tft.setCursor(215,202);  tft.println(rth-1);
  tft.setTextColor(YELLOW);  tft.setCursor(215,202);  tft.println(rth);

 // seconds 
  if ((rts==0) && (rtm>0))
  {
    tft.setTextColor(BLACK); tft.setCursor(287,202);  tft.println("59");
  }
  else
  {
    tft.setTextColor(BLACK);  tft.setCursor(287,202);  tft.println(rts-1);
    tft.setTextColor(YELLOW); tft.setCursor(287,202);  tft.println(rts);
  }
  
//-------------------------------------------
// DISPLAY TOTAL TIME
//-------------------------------------------
  if (rth_EE_99_flag==1)
  {
    if  (RRT_RC == 99)
      {
        tft.setTextColor(BLACK);  tft.setCursor(215,220);  tft.println("0");
        rth_EE_99_flag=0;
      }
      else
      {
        tft.setTextColor(BLACK);  tft.setCursor(215,220);  tft.println("99");
        rth_EE_99_flag=0;  
      }
  }
    
  if (rt_reset == 1)
   {
    tft.setTextColor(BLACK);  tft.setCursor(215,220);  tft.println(rth_EE);
    tft.setTextColor(GREEN);   tft.setCursor(215,220);  tft.println("0");
    rth_EE = 99;  //init value
   }
  
  if (rt_reset == 0)
    {
      tft.setTextColor(BLACK);  tft.setCursor(215,220);  tft.println(rth_EE-1);
      tft.setTextColor(GREEN);   tft.setCursor(215,220);  tft.println(rth_EE);
    }

    
  if (rtm_EE_59_flag==1)
   {
    if  (RRT_RC == 99)
      {
        tft.setTextColor(BLACK);  tft.setCursor(250,220);  tft.println("0");
        rtm_EE_59_flag=0;        
        RRT_RC = 0;
        rt_reset = 0;
      }
      else
      {
        tft.setTextColor(BLACK);  tft.setCursor(250,220);  tft.println("59");
        rtm_EE_59_flag=0;
      }
   }
      
  if (rt_reset == 1)
   {
    tft.setTextColor(BLACK);  tft.setCursor(250,220);  tft.println(rtm_EE);
    tft.setTextColor(GREEN);   tft.setCursor(250,220);  tft.println("0");
    rt_reset = 99;
    rtm_EE = 59;  //init value
    RRT_RC = 99;
   }

  if (rt_reset == 0)
    {
      tft.setTextColor(BLACK);  tft.setCursor(250,220);  tft.println(rtm_EE-1);
      tft.setTextColor(GREEN);   tft.setCursor(250,220);  tft.println(rtm_EE);
    }
}
                                                              
//--------------------------------------------------------
// Read Battery 1-6 voltages
// read battery voltages through serial interface.
//--------------------------------------------------------
void Measure_Battery_Voltages()
{
    bat_voltage_ok=0;
    if(Serial.available()>0)
      {
      //  delay(100); 
        getsercom(Serial.read());    // get command from serial input
      }
}
// ------------------------------------------------------------------------------  
// Check Serial Port for Incoming Characters
// ------------------------------------------------------------------------------  
void getsercom(byte key_command) 
    {
    // B = Sending Battery voltages.
    // format: B,bat1v,bat2v,bat3v,bat4v,bat5v,bat6v,R or C
    // format: B,###,###,###,###,###,###, R or C
    // ------------------------------------------------------------
      if(key_command == 'B')
      {
        BAT1V = Serial.parseInt();      // ###
        BAT2V = Serial.parseInt();      // ###
        BAT3V = Serial.parseInt();      // ###
        BAT4V = Serial.parseInt();      // ###
        BAT5V = Serial.parseInt();      // ###
        BAT6V = Serial.parseInt();      // ###
        RRT_RC = Serial.parseInt();     // 55=Reset, 0=Continue
        BATVend = Serial.parseInt();    // 9
        bat_voltage_ok=1;
      }

    }
//--------------------------------------------------------
// Format Battery 1-6 voltages
//--------------------------------------------------------
void Format_Battery_Voltages()
{
// get float point valves
//BAT1V=945;
BAT1Vf1 = (BAT1V - (BAT1V/100) * 100); //45  945 - (9 *100)
BAT1Vf = BAT1V/100 + (BAT1Vf1/100); //9 + 0.45 = 9.45
BAT2Vf1 = (BAT2V - (BAT2V/100) * 100);
BAT2Vf = BAT2V/100 + (BAT2Vf1/100);
BAT3Vf1 = (BAT3V - (BAT3V/100) * 100);
BAT3Vf = BAT3V/100 + (BAT3Vf1/100);

BAT4Vf1 = (BAT4V - (BAT4V/100) * 100);
BAT4Vf = BAT4V/100 + (BAT4Vf1/100);
BAT5Vf1 = (BAT5V - (BAT5V/100) * 100);
BAT5Vf = BAT5V/100 + (BAT5Vf1/100);
BAT6Vf1 = (BAT6V - (BAT6V/100) * 100);
BAT6Vf = BAT6V/100 + (BAT6Vf1/100);

// Calculate Total Battery Voltage (48v)
TOT_BATVf = BAT1Vf + BAT2Vf + BAT3Vf + BAT4Vf + BAT5Vf + BAT6Vf;

// Calculate Battery Charge (100%)
TOT_BATV_PCT = (TOT_BATVf / 48.00) * 100;
if (TOT_BATV_PCT>100){TOT_BATV_PCT=100;}
}

//--------------------------------------------------------
// Display Battery Voltages
//--------------------------------------------------------
void Display_Battery_Voltages()
{
  DISPLAY_Text();
  tft.setTextColor(BLACK); 
  tft.setCursor(85, 35); tft.println(b1v_prev); 
  tft.setCursor(85, 65); tft.println(b2v_prev);
  tft.setCursor(85, 95); tft.println(b3v_prev);
  tft.setCursor(250, 35); tft.println(b4v_prev);
  tft.setCursor(250, 65); tft.println(b5v_prev);
  tft.setCursor(250, 95); tft.println(b6v_prev);
  b1v_prev= BAT1Vf;
  b2v_prev= BAT2Vf;
  b3v_prev= BAT3Vf;
  b4v_prev= BAT4Vf;
  b5v_prev= BAT5Vf;
  b6v_prev= BAT6Vf;

 //--------------------------------------------------------------------------  
 // ERASE NUMBERS ON LCD.
  tft.setTextColor(BLACK);
  if (t_batpct < 100) {tft.setCursor(235,155);  tft.println(t_batpct);}
  if (t_batpct >= 100){tft.setCursor(225,155);  tft.println(t_batpct);}
 // tft.setCursor(235,155);  tft.println(t_batpct);
  tft.setCursor(235,180);  tft.println(t_batvf);
 //--------------------------------------------------------------------------  
 
  tft.setTextSize(2);
  tft.setCursor(85, 35);
  if (BAT1Vf > 6) {tft.setTextColor(WHITE);}
  if (BAT1Vf <= 6) {tft.setTextColor(RED);}
  tft.println(BAT1Vf); 
   
  tft.setCursor(85, 65);
  if (BAT2Vf > 6) {tft.setTextColor(WHITE);}
  if (BAT2Vf <= 6) {tft.setTextColor(RED);}
  tft.println(BAT2Vf);
  
  tft.setCursor(85, 95);
  if (BAT3Vf > 6) {tft.setTextColor(WHITE);}
  if (BAT3Vf <= 6) {tft.setTextColor(RED);}
  tft.println(BAT3Vf);
  
  tft.setCursor(250, 35);
  if (BAT4Vf > 6) {tft.setTextColor(WHITE);}
  if (BAT4Vf <= 6) {tft.setTextColor(RED);}
  tft.println(BAT4Vf);
  
  tft.setCursor(250, 65);
  if (BAT5Vf > 6) {tft.setTextColor(WHITE);}
  if (BAT5Vf <= 6) {tft.setTextColor(RED);}
  tft.println(BAT5Vf);
  
  tft.setCursor(250, 95);  
  if (BAT6Vf > 6) {tft.setTextColor(WHITE);}
  if (BAT6Vf <= 6) {tft.setTextColor(RED);}
  tft.println(BAT6Vf);
  
  if (TOT_BATV_PCT < 100) {tft.setCursor(235,155);  tft.println(TOT_BATV_PCT);}
  if (TOT_BATV_PCT >= 100){tft.setCursor(225,155);  tft.println(TOT_BATV_PCT);}

  tft.setCursor(235,180);  tft.println(TOT_BATVf);
  bargraph = (210 * TOT_BATV_PCT) / 100;
  tft.fillRect(5, 154, bargraph_prev, 40, BLACK);
  bargraph_prev = bargraph;

  if (TOT_BATV_PCT >= 70) {tft.fillRect(5, 154, bargraph, 40, GREEN);}
  if (TOT_BATV_PCT < 70) {tft.fillRect(5, 154, bargraph, 40, RED);}
  
  t_batpct=TOT_BATV_PCT;
  t_batvf=TOT_BATVf;
}

//--------------------------------------------------------
// POWER UP DISPLAY
//--------------------------------------------------------
unsigned long DISPLAY_INITIAL_Text() 
{
  tft.fillScreen(BLACK);
  tft.fillRect(0, 25, 320, 2, BLUE); 
  tft.setCursor(20, 50);
  tft.setTextColor(GREEN);  tft.setTextSize(4);
  tft.println("CART BATTERY");
  tft.setCursor(80, 90);
  tft.println("MONITOR");
  tft.setCursor(45, 140);
  tft.setTextColor(YELLOW);  tft.setTextSize(2);
  tft.println("BY: ABEtronics.com");
  tft.setCursor(55, 175);
  tft.println("CBM  1-9-16 V1.0");  
  tft.fillRect(0, 210, 320, 2, BLUE); 
  delay(4000);
}

//--------------------------------------------------------
// DISPLAY BATTERY VOLTAGES AND % CHARGE
//--------------------------------------------------------
unsigned long DISPLAY_Text() 
{
  tft.fillScreen(BLACK);
  tft.setCursor(40,10);
  tft.setTextColor(GREEN);  tft.setTextSize(2);
  tft.println("CART BATTERY MONITOR");
  tft.setCursor(1, 35);
  tft.setTextColor(CYAN);  tft.setTextSize(2); 
  tft.println(" BAT1:         BAT4: ");
  tft.setCursor(1, 65);
  tft.println(" BAT2:         BAT5: ");    
  tft.setCursor(1, 95); 
  tft.println(" BAT3:         BAT6: ");    
  
// bottom bar graph. 0-100% charge
  tft.setCursor(10,130);  
  tft.println("--BATTERY CHARGE STATUS--");
  tft.setCursor(305,155);  tft.println("%"); 
  tft.setCursor(305,180);  tft.println("V"); 
  tft.drawRect(3, 152, 214, 44, WHITE);
//  tft.fillRect(5, 188, 210, 40, GREEN); 
// RUN TIME
  tft.setTextColor(CYAN);   tft.setCursor(5,202);    tft.println("RUN TIME (H:M:S)>"); 
  tft.setTextColor(CYAN);   tft.setCursor(237,202);  tft.println(":"); 
  tft.setTextColor(CYAN);   tft.setCursor(275,202);  tft.println(":"); 
  tft.setTextColor(CYAN);   tft.setCursor(5,220);    tft.println("TOT TIME (H:M)  >");
  tft.setTextColor(CYAN);   tft.setCursor(237,220);  tft.println(":"); 
}

void callback()
{
  rts = rts + 1;
  intrup=1;
}
//--------------------------------------------------------

